#include <SPI.h>
#include <Wire.h>
#include <EEPROM.h>
#include "disp.h"
/* Для Adafruit shield это значения по умолчанию.
  Использовать аппаратный SPI (на Uno, #13, #12, #11) и вышеизложенное для CS / DC
  При использовании breakout измените контакты по желанию
  1 TFT_VCC -> 3V3                                                  //через LD01117-3v3 это для моего дисплея
  2 TFT_GND -> GND
  3 TFT_CS  -> D10                                                  //через делитель 1кОм/2кОм
  4 TFT_RST -> RST_pin                                              //на D8 не работает, повесил на RST через резистор 1кОм
  5 TFT_DC  -> D9                                                   //через делитель 1кОм/2кОм
  6 TFT_MOSI-> D11                                                  //через делитель 1кОм/2кОм
  7 TFT_SCK -> D13                                                  //через делитель 1кОм/2кОм
  8 TFT_LED -> 3V3                                                  //через LD01117-3v3
  9 TFT_MISO-> N.C.
  D12-> N.C. */
#include <Adafruit_GFX.h>
#include <Adafruit_ILI9341.h>
#define TFT_DC 9
#define TFT_CS 10
#define Bgcolor 0xC618
#define ILI9341_NAVY            0x000F
#define ILI9341_DARKGREEN       0x03E0
//Adafruit_ILI9341 tft = Adafruit_ILI9341(TFT_CS, TFT_DC);
Adafruit_ILI9341 tft = Adafruit_ILI9341();
#include "Setting.h"
#include "Variables.h"

#ifdef resist_keyboard                                            //если клавиатура резистивная
#include <ClickBtnLong.h>                                         //библиотека аналоговой (резистивной) клавиатуры 
#define PinKeyboard A0                                            //назначаем пин подключения клавиатуры
#define UP_Pin 730                                                //значение на кнопке Up
#define DN_Pin 480                                                //значение на кнопке DOWN
#define ESC_Pin 310                                               //значение на кнопке LEFT
#define RT_Pin 120                                                //значение на кнопке RIGHT
#define OK_Pin 0                                                  //значение на кнопке OK
#define ABERRATION 30
ClickBtnLong Btn_UP(PinKeyboard, UP_Pin, ABERRATION);             //передаём данные кнопок резистивной клавиатуры в библиотеку ClickBtnLong.h
ClickBtnLong Btn_DOWN(PinKeyboard, DN_Pin, ABERRATION);           //
ClickBtnLong Btn_LEFT(PinKeyboard, ESC_Pin, ABERRATION);          //
ClickBtnLong Btn_RIGHT(PinKeyboard, RT_Pin, ABERRATION);          //
ClickBtnLong Btn_OK(PinKeyboard, OK_Pin, ABERRATION);             //
#else
#include <s_button.h>                                             //обработчик кнопок (button_5)
int holdTime = 2000;                                              //msec длительность нажатия + удержания
int debounce = 60;                                                //msec антидребезг
SButton UP_KEY (UP_Pin,  debounce);
SButton DN_KEY (DN_Pin,  debounce);
SButton ESC_KEY(ESC_Pin, debounce, holdTime);
SButton OK_KEY (OK_Pin,  debounce, holdTime);
#endif

void setup() {
  Serial.begin(9600);
#ifdef resist_keyboard
  pinMode(A0, INPUT);
#else
  UP_KEY.begin();
  DN_KEY.begin();
  ESC_KEY.begin();
  OK_KEY.begin();
#endif
  pinMode(buzzerPin, OUTPUT);
  pinMode(RelayPin1, OUTPUT);
  pinMode(RelayPin2, OUTPUT);
  pinMode(2, INPUT);                                               //ZCC
  pinMode(thermoCStop, OUTPUT);
  pinMode(thermoCSbott, OUTPUT);
  pinMode(thermoCLK, OUTPUT);
  pinMode(thermoSO, INPUT);

  HelloGuys();                                                     //сообщение приветствия
  SongHello();                                                     //Мелодия приветствия
  delay(3000);
  loadProfile();                                                   //вызов функции loadProfile для загрузки данных профиля из eeprom
  //windowStartTime = millis();
  nextRead1 = millis();

  /*настраиваем работу с прерыванием: LOW вызывает прерывание, когда на порту LOW
    CHANGE прерывание вызывается при смене значения на порту, с LOW на HIGH и наоборот
    RISING прерывание вызывается только при смене значения на порту с LOW на HIGH
    FALLING прерывание вызывается только при смене значения на порту с HIGH на LOW */
#ifdef SetInterrupt
  attachInterrupt(SetInterrupt, Dimming, RISING);                  //настроить порт прерывания(0 или 1) 2й или 3й цифровой пин
  //attachInterrupt(SetInterrupt, Dimming, FALLING);                //настроить порт прерывания(0 или 1) 2й или 3й цифровой пин
#else
  MsTimer2::set(10, Dimming);                                      //100ms period
  MsTimer2::start();
#endif
}

char buf[40];                                                      //стандартный буфер, если станцию настроили

void loop() {
  unsigned long currentMillis = millis();
  int SP2 = profile.Setpoint2;
  float out_float;                                                 //вывод на tft lcd дробных чисел
  reflow_complete_ok();

#ifdef resist_keyboard
  Btn_UP.run(&PushUp, &PushUpLong);                   //указываем обработчики для кнопки Up
  Btn_DOWN.run(&PushDown, &PushDownLong);             //указываем обработчики для кнопки Down
  Btn_LEFT.run(&PushLeft, &PushLeftLong);             //указываем обработчики для кнопки Left
  Btn_RIGHT.run(&PushRight, &PushRightLong);          //указываем обработчики для кнопки Right
  Btn_OK.run(&PushOk, &PushOkLong);                   //указываем обработчики для кнопки Ok
#endif

  switch (reflowState) {
    case REFLOW_STATE_IDLE: {
        TopStart = false;
        currentStep = 1;
        counter = 0;
        setpointRamp = 0;
        x = 1;                                                     //устанавливаем переменную в исходное состояние
        flag = 0;                                                  //после отановки профиля сбрасываем флаг
        Output1 = 0;
        Output2 = 0;
        gx = 21;

        if (millis() > nextRead1) {                                //чтение и вывод на LCD температур, подготовка данных для ПК
          nextRead1 = millis() + SENSOR_SAMPLING_TIME;
          TempRead();
#ifdef pcgr
#ifdef SerialPortPlotter
          sprintf (buf, "$%03d %03d %03d %03d;", int(Output1), int(Output2), tc1, tc2); // стандартный вывод данных
#else
          sprintf (buf, "OK%03d%03d%03d%03d%03d\r\n", int(Output1), int(Output2), tc1, tc2, currentProfile);
#endif
#endif
        }

        if (updateScreen) {                                        //Настройка экрана в режиме ожидания
          cls();
          tft.setTextColor(ILI9341_RED, Bgcolor);  tft.setTextSize(fontsize3);
          tft.setCursor(col(mode_x), row(mode_y));
          tft.print("IDLE");                                       //индикация режима ожидания
          tft.setCursor(col(1), row(4));
          tft.print("PTN:");                                       //номер профиля
          tft.setCursor(col(5), row(4));
          tft.print(currentProfile);                               //номер выбранного профиля
          tft.setCursor(col(9), row(4));
          tft.print("STEPS:");                                     //число шагов ВИ в профиле
          tft.setCursor(col(15), row(4));
          tft.print(profile.profileSteps);                         //число шагов ВИ в выбранном профиле
          //tft.setCursor(col(10), row(4));
          //tft.print("STEP:"); tft.print("1");//currentStep
          tft.setCursor(col(1), row(6));
          tft.print("TH=");                                        //уставка первого шага ВИ выбранного профиля
          tft.setCursor(col(4), row(6));
          tft.print(profile.temperatureStep[currentProfile - 1]);                   // ??? значение уставки первого шага ВИ выбранного профиля
          // tft.print("  ");
          tft.setCursor(col(10), row(6));
          tft.print(char(30)); tft.print("T=");                    //температура на датчике ВИ
          tft.setCursor(col(1), row(8));
          tft.print("BH=");                                        //уставка НИ выбранного профиля
          tft.setCursor(col(10), row(8));
          tft.print(char(31)); tft.print("T=");                    //температура на датчике НИ
          tft.setCursor(col(4), row(8));
          tft.print(SP2); tft.print(" ");                          //значение уставки НИ выбранного профиля
          updateScreen = false;
        }
        tft.setTextColor(ILI9341_RED, Bgcolor);  tft.setTextSize(fontsize3);
        tft.setCursor(col(5), row(4));
        tft.print(currentProfile);                                 //номер выбранного профиля
        tft.setCursor(col(4), row(6));
        tft.print(profile.temperatureStep[currentProfile - 1]);    // ???
        //tft.print(profile.temperatureStep[0]);
        tft.print("  ");
        tft.setCursor(col(4), row(8));
        tft.print(SP2); tft.print("  ");
        tft.setCursor(col(tc_x), row(tc1_y));
        tft.print(tc1); tft.print("  ");                           //реальная температура на датчике ВИ
        tft.setCursor(col(tc_x), row(tc2_y));
        tft.print(tc2); tft.print("  ");                           //реальная температура на датчике НИ
        //windowStartTime = millis();
#ifndef  resist_keyboard
        if ( UP_KEY.isPressed() == SB_CLICK) {
          beeper();
          if (++currentProfile >= 5)  currentProfile = 1;
        }
        if ( DN_KEY.isPressed() == SB_CLICK) {
          beeper();
          if (--currentProfile <= 0)  currentProfile = 4;
        }

        switch ( OK_KEY.isPressed() ) {                            //фиксируем момент нажатия кнопки "ОК" + защита от дребезга
          case SB_CLICK: {                                         //старт профиля
              tone(buzzerPin, 1045, 500);  //звуковой сигнал при старте профиля
              updateScreen = true;
#ifdef pcgr
#ifndef SerialPortPlotter
              sprintf (buf, "SYNC\r\n");
              //Serial.print(buf);
#endif
#endif
              curCount = 0;
              nextRead1 = millis();
              nextRead2 = millis();
              integra = 0;
              integra2 = 0;
              p1 = 0;
              p2 = 0;
              d1 = 0;
              d2 = 0;
              previousMillis = millis();
              reflowStatus = REFLOW_STATUS_ON;
              reflowState = REFLOW_STATE_STEP_RAMP;
            } break;
          case SB_LONG_CLICK: {                                    //меню, пункт настройки уставки НИ
              beeper();
              reflowState = REFLOW_STATE_MENU_BOTTOM_HEAT;
              updateScreen = true;
            } break;
        }
        switch ( ESC_KEY.isPressed() ) {
          case SB_CLICK: {                                         //что-то делаем при нажатии ESC
              beeper();
            } break;
          case SB_LONG_CLICK: {                                    //делаем инит встроенного профиля при долгом нажатии ESC
              beeper();
              reflowState = REFLOW_STATE_PROFILE_INIT;
              updateScreen = true;
            }  break;
        }
#endif
      } break;

    case REFLOW_STATE_PROFILE_INIT: {                            //инициализация тестового профиля
        /* 4 шага,
           уставка НИ 160°,
           BOTTOM PWR 50%,
           TOP PWR 40%,
           скорости роста t° по шагам 10,3,3,6,
           уставки ВИ по шагам 180,190,200,225,
           длительности полок по шагам 10,15,15,30,
           P.I.D BOTTOM 8,20,40, P.I.D TOP 10,15,40 */

        profile = {4, 160, 50, 40, 10, 3, 3, 6, 180, 190, 200, 225, 10, 15, 15, 30, 10, 15, 40, 8, 20, 40};
        SaveProfile();
        tone(buzzerPin, 800, 200);                                 //звуковой сигнал при сбросе профиля
        delay(2000);
        updateScreen = true;
        reflowState = REFLOW_STATE_IDLE;
      } break;
    
    case REFLOW_STATE_MENU_BOTTOM_HEAT: {                          //устанавливаем температуру "Нижнего Нагревателя"
        if (updateScreen) {
          cls();
          tft.setTextColor(ILI9341_RED, Bgcolor);  tft.setTextSize(fontsize3);
          tft.setCursor(col(mode_x), row(mode_y));                 //x=7,y=2
          tft.print("SETUP");
          tft.setCursor(col(1), row(6));
          tft.print("Bottom T=         ");
          updateScreen = false;
        }
        tft.setTextColor(ILI9341_RED, Bgcolor);  tft.setTextSize(fontsize3);
        tft.setCursor(col(10), row(6));
        tft.print(SP2); tft.print(char(128)); tft.print("C  ");

#ifndef  resist_keyboard
        switch (UP_KEY.isPressed()) {
          case SB_CLICK: {
              beeper();
              profile.Setpoint2 += 5;
              if (profile.Setpoint2 >= 255) profile.Setpoint2 = 255;
            } break;
          case SB_AUTO_CLICK: {
              profile.Setpoint2 += 5;
              if (profile.Setpoint2 >= 255) profile.Setpoint2 = 255;
            } break;
        }
        switch (DN_KEY.isPressed()) {
          case SB_CLICK: {
              beeper();
              profile.Setpoint2 -= 5;
              if (profile.Setpoint2 <= 80) profile.Setpoint2 = 80;
            } break;
          case SB_AUTO_CLICK: {
              profile.Setpoint2 -= 5;
              if (profile.Setpoint2 <= 80) profile.Setpoint2 = 80;
            } break;
        }
        if ( OK_KEY.isPressed() == SB_CLICK)  {
          beeper();
          updateScreen = true;
          SaveProfile();
          reflowState = REFLOW_STATE_MENU_BOTTOM_PWR;
        }
        if ( ESC_KEY.isPressed() == SB_CLICK) {
          CancelState();
        }
#endif
      } break;

    case REFLOW_STATE_MENU_BOTTOM_PWR: {                           //устанавливаем максимальную мощность "НИЖНЕГО НАГРЕВАТЕЛЯ"
        if (updateScreen) {
          tft.setTextColor(ILI9341_RED, Bgcolor);  tft.setTextSize(fontsize3);
          tft.setCursor(col(1), row(6));
          tft.print("Bottom PWR=      ");
          updateScreen = false;
        }
        tft.setTextColor(ILI9341_RED, Bgcolor);  tft.setTextSize(fontsize3);
        tft.setCursor(col(12), row(6));
        tft.print(profile.pwr_BOTTOM);  tft.print("%  ");

#ifndef  resist_keyboard
        switch (UP_KEY.isPressed()) {
          case SB_CLICK: {
              beeper();
              if (++profile.pwr_BOTTOM >= 99) profile.pwr_BOTTOM = 99;
            } break;
          case SB_AUTO_CLICK: {
              if (++profile.pwr_BOTTOM >= 99) profile.pwr_BOTTOM = 99;
            } break;
        }
        switch (DN_KEY.isPressed()) {
          case SB_CLICK: {
              beeper();
              if (--profile.pwr_BOTTOM <= 0) profile.pwr_BOTTOM = 0;
            }  break;
          case SB_AUTO_CLICK: {
              if (--profile.pwr_BOTTOM <= 0) profile.pwr_BOTTOM = 0;
            } break;
        }
        if ( OK_KEY.isPressed() == SB_CLICK) {
          beeper();
          updateScreen = true;
          SaveProfile();
          reflowState = REFLOW_STATE_MENU_TOP_PWR;
        }
        if ( ESC_KEY.isPressed() == SB_CLICK) {
          CancelState();
        }
#endif
      }  break;

    case REFLOW_STATE_MENU_TOP_PWR: {                              //устанавливаем максимальную мощность "ВЕРХНЕГО НАГРЕВАТЕЛЯ"
        if (updateScreen) {
          tft.setTextColor(ILI9341_RED, Bgcolor);
          tft.setCursor(col(1), row(6));
          tft.print("   TOP PWR=         ");
          updateScreen = false;
        }
        tft.setTextColor(ILI9341_RED, Bgcolor);
        tft.setCursor(col(12), row(6));
        tft.print(profile.pwr_TOP);  tft.print("%  ");

#ifndef  resist_keyboard
        switch (UP_KEY.isPressed()) {
          case SB_CLICK: {
              beeper();
              if (++profile.pwr_TOP >= 99) profile.pwr_TOP = 99;
            } break;
          case SB_AUTO_CLICK: {
              if (++profile.pwr_TOP >= 99) profile.pwr_TOP = 99;
            } break;
        }
        switch (DN_KEY.isPressed()) {
          case SB_CLICK: {
              beeper();
              if (--profile.pwr_TOP <= 0) profile.pwr_TOP = 0;
            } break;
          case SB_AUTO_CLICK: {
              if (--profile.pwr_TOP <= 0) profile.pwr_TOP = 0;
            } break;
        }
        if ( OK_KEY.isPressed() == SB_CLICK) {
          beeper();
          updateScreen = true;
          SaveProfile();
          reflowState = REFLOW_STATE_MENU_STEPS;
        }
        if ( ESC_KEY.isPressed() == SB_CLICK) {
          CancelState();
        }
#endif
      }  break;

case REFLOW_STATE_MENU_STEPS: {                                //устанавливаем количество шагов профиля
        if (updateScreen) {
          tft.setTextColor(ILI9341_RED, Bgcolor);  tft.setTextSize(fontsize3);
          tft.setCursor(col(1), row(4));
          tft.print("Profile:");
          tft.setCursor(col(9), row(4));
          tft.print(currentProfile);
          tft.setCursor(col(11), row(4));
          tft.print("Edit   ");
          tft.setCursor(col(1), row(6));
          tft.print("Profile Steps:    ");
          updateScreen = false;
        }
        tft.setTextColor(ILI9341_RED, Bgcolor);  tft.setTextSize(fontsize3);
        tft.setCursor(col(15), row(6));
        tft.print(profile.profileSteps);

#ifndef  resist_keyboard
        if ( UP_KEY.isPressed() == SB_CLICK) {
          beeper();
          if (++profile.profileSteps >= 5) profile.profileSteps = 1;
        }
        if ( DN_KEY.isPressed() == SB_CLICK) {
          beeper();
          if (--profile.profileSteps <= 0)  profile.profileSteps = 4;
        }
        if ( OK_KEY.isPressed() == SB_CLICK) {
          beeper();
          updateScreen = true;
          SaveProfile();
          reflowState = REFLOW_STATE_MENU_STEP_TARGET;
        }
        if ( ESC_KEY.isPressed() == SB_CLICK) {
          CancelState();
        }
#endif
      } break;

    case REFLOW_STATE_MENU_STEP_TARGET: {                          //устанавливаем температуру "Верхнего Нагревателя"
        if (updateScreen) {
          tft.setTextColor(ILI9341_RED, Bgcolor);  tft.setTextSize(fontsize3);
          tft.setCursor(col(1), row(4));
          tft.print("SET TOP TEMP      ");
          tft.setCursor(col(1), row(6));
          tft.print("Step:  TEMP:      ");
          tft.setCursor(col(6), row(6));
          tft.print(editStep + 1);
          tft.setCursor(col(13), row(6));
          tft.print(profile.temperatureStep[editStep]);
          tft.print("  ");
          updateScreen = false;
        }
        tft.setTextColor(ILI9341_RED, Bgcolor);
        tft.setCursor(col(13), row(6));
        tft.print(profile.temperatureStep[editStep]);
        tft.print("  ");

#ifndef  resist_keyboard
        switch (UP_KEY.isPressed()) {
          case SB_CLICK: {
              beeper();
              if (++profile.temperatureStep[editStep] >= 255) profile.temperatureStep[editStep] = 255;
            } break;
          case SB_AUTO_CLICK: {
              if (++profile.temperatureStep[editStep] >= 255) profile.temperatureStep[editStep] = 255;
            } break;
        }
        switch (DN_KEY.isPressed()) {
          case SB_CLICK: {
              beeper();
              if (--profile.temperatureStep[editStep] <= 80) profile.temperatureStep[editStep] = 80;
            }  break;
          case SB_AUTO_CLICK: {
              if (--profile.temperatureStep[editStep] <= 80) profile.temperatureStep[editStep] = 80;
            } break;
        }
        if ( OK_KEY.isPressed() == SB_CLICK) {
          beeper();
          updateScreen = true;
          SaveProfile();
          if (editStep + 1 == profile.profileSteps) {
            editStep = 0;
            reflowState = REFLOW_STATE_MENU_STEP_RAMP;
          }  else {
            editStep++;
          }
        }
        if ( ESC_KEY.isPressed() == SB_CLICK) {
          editStep = 0;
          CancelState();
        }
#endif
      }  break;

    case REFLOW_STATE_MENU_STEP_RAMP: {                            //устанавливаем скорость нагрева "Верхним Нагревателем"
        if (updateScreen) {
          tft.setTextColor(ILI9341_RED, Bgcolor);  tft.setTextSize(fontsize3);
          tft.setCursor(col(1), row(4));
          tft.print("HEAT speed TOP   ");
          tft.setCursor(col(1), row(6));
          tft.print("Step:  Ramp:       ");
          tft.setCursor(col(11), row(7)); tft.print(char(128));
          tft.print("C/sec");
          tft.setCursor(col(6), row(6));
          tft.print(editStep + 1);
          updateScreen = false;
        }
        tft.setTextColor(ILI9341_RED, Bgcolor);
        tft.setCursor(col(13), row(6));
        out_float = profile.rampRateStep[editStep] / 10.0;
        tft.print(out_float, 1);
        tft.print("  ");

#ifndef  resist_keyboard
        if ( UP_KEY.isPressed() == SB_CLICK) {
          beeper();
          if (++profile.rampRateStep[editStep] >= 30)  profile.rampRateStep[editStep] = 30;
        }
        if ( DN_KEY.isPressed() == SB_CLICK) {
          beeper();
          if (--profile.rampRateStep[editStep] <= 1)  profile.rampRateStep[editStep] = 1;
        }
        if ( OK_KEY.isPressed() == SB_CLICK) {
          beeper();
          updateScreen = true;
          if (editStep + 1 == profile.profileSteps) {
            editStep = 0;
            reflowState = REFLOW_STATE_MENU_STEP_DWELL;
          }  else editStep++;
          SaveProfile();
        }
        if ( ESC_KEY.isPressed() == SB_CLICK) {
          CancelState();
        }
#endif
      }  break;

    case REFLOW_STATE_MENU_STEP_DWELL: {                           //устанавливаем время перехода на следующий шаг
        if (updateScreen) {
          tft.setTextColor(ILI9341_RED, Bgcolor);  tft.setTextSize(fontsize3);
          tft.setCursor(col(11), row(7)); tft.print("      ");
          tft.setCursor(col(0), row(4));
          tft.print(" Time next step   ");
          tft.setCursor(col(1), row(6));
          tft.print("Step:"); tft.print(editStep + 1);
          tft.print(" Time:"); tft.print(profile.dwellTimerStep[editStep]);
          tft.print("sec  ");
        }
        tft.setTextColor(ILI9341_RED, Bgcolor);  tft.setTextSize(fontsize3);
        tft.setCursor(col(13), row(6));
        tft.print(profile.dwellTimerStep[editStep]);
        tft.print("sec  ");
        //tft.print(" ");

#ifndef  resist_keyboard
        if ( UP_KEY.isPressed() == SB_CLICK) {
          beeper();
          profile.dwellTimerStep[editStep] = profile.dwellTimerStep[editStep] + 5;
          if (profile.dwellTimerStep[editStep] >= 255) profile.dwellTimerStep[editStep] = 255;
        }
        if ( DN_KEY.isPressed() == SB_CLICK) {
          beeper();
          profile.dwellTimerStep[editStep] = profile.dwellTimerStep[editStep] - 5;
          if (profile.dwellTimerStep[editStep] <= 0)  profile.dwellTimerStep[editStep] = 0;
        }
        if ( OK_KEY.isPressed() == SB_CLICK) {
          beeper();
          updateScreen = true;
          SaveProfile();
          if (editStep + 1 == profile.profileSteps) {
            editStep = 0;
            reflowState = REFLOW_STATE_MENU_BOTTOM_P;
          } else {
            editStep++;
          }
        }
        if ( ESC_KEY.isPressed() == SB_CLICK) {
          editStep = 0;
          CancelState();
        }
#endif
      }  break;

    case REFLOW_STATE_MENU_BOTTOM_P: {                             //настройка "П" ПИД нижнего нагревателя
        if (updateScreen) {
          tft.setTextColor(ILI9341_RED, Bgcolor);  tft.setTextSize(fontsize3);
          tft.setCursor(0 * 16, 1 * 48 + 48);
          tft.print("        PID        ");
          tft.setCursor(0 * 16 + 5, 2 * 48 + 48);
          tft.print("                  ");
          tft.setCursor(1 * 16, 2 * 48 + 48);
          tft.print("Bottom Heater:");
          tft.setCursor(8 * 16, 3 * 48 + 48);
          tft.print("P= ");//lcd.print("P=");
          tft.setCursor(11 * 16, 3 * 48 + 48);
          tft.print(profile.kp2);
          tft.print("  ");
          updateScreen = false;
        }
        tft.setTextColor(ILI9341_RED, Bgcolor);  tft.setTextSize(fontsize3);
        tft.setCursor(11 * 16, 3 * 48 + 48);
        tft.print(profile.kp2);
        tft.print("  ");

        profile.kp2 = set_kPID(profile.kp2);

#ifndef  resist_keyboard
        if ( OK_KEY.isPressed() == SB_CLICK) {
          beeper();
          updateScreen = true;
          SaveProfile();
          reflowState = REFLOW_STATE_MENU_BOTTOM_I;
        }
        if ( ESC_KEY.isPressed() == SB_CLICK) {
          CancelState();
        }
      }  break;
    case REFLOW_STATE_MENU_BOTTOM_I: {                             //настройка "И" ПИД нижнего нагревателя
        tft.setCursor(0 * 16, 1 * 48 + 48);
        tft.print("        PID        ");
        tft.setCursor(8 * 16, 3 * 48 + 48);
        tft.print("I= ");
        tft.setCursor(11 * 16, 3 * 48 + 48);
        tft.print(profile.ki2);
        tft.print(" ");

        profile.ki2 = set_kPID(profile.ki2);

        if ( OK_KEY.isPressed() == SB_CLICK) {
          beeper();
          SaveProfile();
          reflowState = REFLOW_STATE_MENU_BOTTOM_D;
        }
        if ( ESC_KEY.isPressed() == SB_CLICK) {
          CancelState();
        }
#endif
      }  break;

    case REFLOW_STATE_MENU_BOTTOM_D: {                             //настройка "Д" ПИД нижнего нагревателя
        tft.setCursor(0 * 16, 1 * 48 + 48);
        tft.print("        PID        ");
        tft.setCursor(8 * 16, 3 * 48 + 48);
        tft.print("D= ");
        tft.setCursor(11 * 16, 3 * 48 + 48);
        tft.print(profile.kd2);
        tft.print(" ");

        profile.kd2 = set_kPID(profile.kd2);

#ifndef  resist_keyboard
        if ( OK_KEY.isPressed() == SB_CLICK) {
          beeper();
          SaveProfile();
          updateScreen = true;
          reflowState = REFLOW_STATE_MENU_TOP_P;
        }
        if ( ESC_KEY.isPressed() == SB_CLICK) {
          CancelState();
        }
#endif
      }  break;

    case REFLOW_STATE_MENU_TOP_P: {                                //настройка "П" ПИД верхнего нагревателя
        if (updateScreen) {
          tft.setTextColor(ILI9341_RED, Bgcolor);  tft.setTextSize(fontsize3);
          tft.setCursor(0 * 16, 1 * 48 + 48);
          tft.print("        PID        ");
          tft.setCursor(0 * 16 + 5, 2 * 48 + 48);
          tft.print("                  ");
          tft.setCursor(3 * 16, 2 * 48 + 48);
          tft.print("TOP Heater:   ");
          tft.setCursor(8 * 16, 3 * 48 + 48);
          tft.print("P= ");
          tft.setCursor(11 * 16, 3 * 48 + 48);
          tft.print(profile.kp1);
          tft.print(" ");
          updateScreen = false;
        }
        tft.setTextColor(ILI9341_RED, Bgcolor);  tft.setTextSize(fontsize3);
        tft.setCursor(11 * 16, 3 * 48 + 48);
        tft.print(profile.kp1);
        tft.print("  ");

        profile.kp1 = set_kPID(profile.kp1);

#ifndef  resist_keyboard
        if ( OK_KEY.isPressed() == SB_CLICK) {
          beeper();
          SaveProfile();
          reflowState = REFLOW_STATE_MENU_TOP_I;
        }
        if ( ESC_KEY.isPressed() == SB_CLICK) {
          CancelState();
        }
#endif
      } break;

    case REFLOW_STATE_MENU_TOP_I: {                                //настройка "И" ПИД верхнего нагревателя
        tft.setCursor(0 * 16, 1 * 48 + 48);
        tft.print("        PID        ");
        tft.setCursor(8 * 16, 3 * 48 + 48);
        tft.print("I= ");
        tft.setCursor(11 * 16, 3 * 48 + 48);
        tft.print(profile.ki1);
        tft.print(" ");

        profile.ki1 = set_kPID(profile.ki1);

#ifndef  resist_keyboard
        if ( OK_KEY.isPressed() == SB_CLICK) {
          beeper();
          SaveProfile();
          reflowState = REFLOW_STATE_MENU_TOP_D;
        }
        if ( ESC_KEY.isPressed() == SB_CLICK) {
          CancelState();
        }
#endif
      } break;

    case REFLOW_STATE_MENU_TOP_D: {                                //настройка "Д" ПИД верхнего нагревателя
        tft.setCursor(0 * 16, 1 * 48 + 48);
        tft.print("        PID        ");
        tft.setCursor(8 * 16, 3 * 48 + 48);
        tft.print("D=");
        tft.setCursor(11 * 16, 3 * 48 + 48);
        tft.print(profile.kd1);
        tft.print(" ");

        profile.kd1 = set_kPID(profile.kd1);

#ifndef  resist_keyboard
        if ( OK_KEY.isPressed() == SB_CLICK) {
          beeper();
          SaveProfile();
          updateScreen = true;
          reflowState = REFLOW_STATE_IDLE;
        }
        if ( ESC_KEY.isPressed() == SB_CLICK) {
          CancelState();
        }
#endif
      } break;

    case REFLOW_STATE_PRE_HEATER: {
        Output2 = 3;
        if (millis() - previousMillis > 5000) {
          Output2 = 0;
          reflowState = REFLOW_STATE_STEP_RAMP;
        }

#ifndef  resist_keyboard
        if ( ESC_KEY.isPressed() == SB_CLICK) {
          Output2 = 0;
          beeper();
          updateScreen = true;
          reflowState = REFLOW_STATE_COMPLETE;
        }
#endif
      } break;

    case REFLOW_STATE_STEP_RAMP: {                                 //"процесс пайки", рост температуры с заданной скоростью
        if (updateScreen) {
          tft.setTextColor(ILI9341_GREEN, Bgcolor); tft.setTextSize(fontsize3);
          tft.setCursor(col(mode_x), row(mode_y));
          tft.print("       ");
          //#ifdef no_pc
          prn_gr();
          //#endif
          tft.setTextColor(ILI9341_BLACK, Bgcolor); tft.setTextSize(1);
          tft.setCursor(col_1(4), row_1(6));
          tft.print("PTN:"); tft.print(currentProfile); tft.print("  ");
          tft.print("Step:"); tft.print(currentStep);
          tft.print("/"); tft.print(profile.profileSteps);
          tft.setCursor(col_1(4), row_1(7));
          tft.print("SPramp="); tft.print(setpointRamp);
          tft.setCursor(col_1(4), row_1(8));
          tft.print("Ramp=");
          //tft.print(profile.rampRateStep[currentStep - 1]);
          out_float = profile.rampRateStep[editStep] / 10.0;
          tft.print(out_float, 1);
          tft.print(" ");
          tft.print(char(128)); tft.print("C/sec");
          updateScreen = false;
        }
        tft.setTextColor(ILI9341_BLACK, Bgcolor); tft.setTextSize(1);
        tft.setCursor(col_1(4), row_1(10)); //25,82
        tft.setTextColor(ILI9341_BLUE, Bgcolor); tft.setTextSize(1);
        tft.print("BOTTOM="); tft.print(tc2); tft.print("/");
        tft.print(SP2); tft.print(char(128)); tft.print("C ");
        tft.print("PWM="); tft.print(profile.pwr_BOTTOM); tft.print("%  ");
        tft.setTextColor(ILI9341_RED, Bgcolor); tft.setTextSize(1);
        tft.print("TOP="); tft.print(tc1); tft.print("/");
        tft.print(profile.temperatureStep[currentStep - 1]); tft.print(char(128)); tft.print("C ");
        tft.print("PWM="); tft.print(profile.pwr_TOP); tft.print("% ");
        tft.setCursor(col_1(4), row_1(7));
        tft.print("SPramp="); tft.print(setpointRamp);
        edit_pid();

        if (tc1 >= SP2 - 10 && !TopStart) TopStart = true;         //если по данным ДАТЧИКА У ЧИПА низ достиг уставки, или уставки минус 20°С, включаем верхний нагреватель
        if (TopStart == true) {                                    //открываем скобку (чтоб не запутаться) //если включен верхний нагреватель
          if (flag == 0) {                                         //фиксируем стартовую температуру
            startTemp = tc1;
            flag = 1;
          }
          if (startTemp > profile.temperatureStep[currentStep - 1]) {  //устанавливаем нужный шаг, до которого нагрета плата
            for (x = 1; startTemp > profile.temperatureStep[currentStep - 1]; currentStep++) {
              x++;
            }
          }
          if (currentStep > x && flag == 1) {
            flag = 0;
            startTemp = profile.temperatureStep[currentStep - 2];
            flag = 1;
          }
          if ((currentMillis - previousMillis) > 1000 / (profile.rampRateStep[currentStep - 1] * 0.1))  {  //счётчик скорости роста температуры от 0.1с. до 3с.
            previousMillis = currentMillis;
            counter = counter + 1; // counter++;
            setpointRamp = counter + startTemp;
            Setpoint1 = setpointRamp;
          }
        }                                                          //закрывам скобку

        if (setpointRamp >= profile.temperatureStep[currentStep - 1]) { //если достигли нужной температуры
          reflowState = REFLOW_STATE_STEP;
        }

#ifndef  resist_keyboard
        if ( ESC_KEY.isPressed() == SB_CLICK) {
          beeper();
          updateScreen = true;
          reflowStatus = REFLOW_STATUS_OFF;
          reflowState = REFLOW_STATE_IDLE;
        }
#endif
      }  break;

    case REFLOW_STATE_STEP: {
        Setpoint1 = profile.temperatureStep[currentStep - 1];
        if (Input1 >= profile.temperatureStep[currentStep - 1]) {
          counter = 0;
          reflowState = REFLOW_STATE_STEP_DWELL;
        }

#ifndef  resist_keyboard        
        if ( ESC_KEY.isPressed() == SB_CLICK) {
          beeper();
          updateScreen = true;
          reflowStatus = REFLOW_STATUS_OFF;
          reflowState = REFLOW_STATE_IDLE;
        }
#endif
      } break;

    case REFLOW_STATE_STEP_DWELL: {                                //считаем время перехода на следующий шаг
        if (currentMillis - previousMillis > 1000) {
          previousMillis = currentMillis;
          counter = counter + 1;  // counter++;
        }
        if (counter == profile.dwellTimerStep[currentStep - 1]) {  //если счётчик равен установленному времени
          tone(buzzerPin, 1045, 500);                              //звуковой сигнал
          counter = 0;
          setpointRamp = 0;
          if (profile.profileSteps == currentStep) {               //если достигли последнего шага
            currentStep = 1;
            x = 1;                                                 //устанавливаем переменную в исходное состояние
            flag = 0;                                              //после завершения профиля сбрасываем флаг
            reflowState = REFLOW_STATE_COMPLETE;
          } else  {                                                //если шаг не последний
            currentStep++;                                         //переходим на следующий шаг
            reflowState = REFLOW_STATE_STEP_RAMP;
          }
        }
 
#ifndef  resist_keyboard        
        if ( ESC_KEY.isPressed() == SB_CLICK) {
          beeper();
          updateScreen = true;
          reflowStatus = REFLOW_STATUS_OFF;
          reflowState = REFLOW_STATE_IDLE;
        }
#endif
      } break;

    case REFLOW_STATE_COMPLETE: {                                  //завершение пайки
        reflowStatus = REFLOW_STATUS_OFF;
        reflowState = REFLOW_STATE_IDLE;
        updateScreen = true;
      } break;

  }                                                                //здесь закончился switch

  if (reflowStatus == REFLOW_STATUS_ON) {                          //включение нагревателей
    if (millis() > nextRead1) {
      nextRead1 = millis() + SENSOR_SAMPLING_TIME;                 //считываем данные с термопар и считаем ПИД
      if (reflowState != REFLOW_STATE_PRE_HEATER) Output2 = Pid2(Input2, SP2, profile.kp2, profile.ki2, profile.kd2);
      if (TopStart) Output1 = Pid1(Input1, Setpoint1, profile.kp1, profile.ki1, profile.kd1); else Output1 = 0;
      //Output2 = Pid2(Input2, SP2, profile.kp2, profile.ki2, profile.kd2);

      TempRead();

      gy1 = map(tc1_gr, 0, 300, h - 1, 91);
      gy2 = map(tc2_gr, 0, 300, h - 1, 91);
      tft.drawLine(gx - 1, gy1_prev, gx, gy1, ILI9341_RED);
      tft.drawLine(gx - 1, gy2_prev, gx, gy2, ILI9341_BLUE);
      gy1_prev = gy1;
      gy2_prev = gy2;
      gx++;
      if (gx > (w - 2)) {
        gx = 21;
        tft.fillRect(0, 90, w, h, Bgcolor);//cls

        prn_gr();
      }
      tft.setTextColor(ILI9341_BLACK, Bgcolor); tft.setTextSize(1);
      tft.setCursor(col_1(4), row_1(10)); //25,82
      tft.setTextColor(ILI9341_BLUE, Bgcolor); tft.setTextSize(1);
      tft.print("BOTTOM="); tft.print(tc2); tft.print("/");
      tft.print(SP2); tft.print(char(128)); tft.print("C ");
      tft.print("PWM="); tft.print(profile.pwr_BOTTOM); tft.print("%  ");
      tft.setTextColor(ILI9341_RED, Bgcolor); tft.setTextSize(1);
      tft.print("TOP="); tft.print(tc1); tft.print("/");
      tft.print(profile.temperatureStep[currentStep - 1]); tft.print(char(128)); tft.print("C ");
      tft.print("PWM="); tft.print(profile.pwr_TOP); tft.print("% ");
      tft.setCursor(col_1(4), row_1(7));
      tft.print("SPramp=");
      tft.print(setpointRamp);
    }
    if (millis() > nextRead2) {                                    //готовим данные для вывода на ПК
      nextRead2 = millis() + GRAPHICS_SAMPLING_TIME;
#ifndef no_pcgr
#ifdef SerialPortPlotter
      sprintf (buf, "$%03d %03d %03d %03d;", int(Output1), int(Output2), tc1, tc2); // стандартный вывод данных
#else
      sprintf (buf, "OK%03d%03d%03d%03d%03d\r\n", int(Output1), int(Output2), tc1, tc2, currentProfile);
#endif
#endif
    }
  }  else {
    digitalWrite(RelayPin1, LOW);
    digitalWrite(RelayPin2, LOW);
  }

}                                                                  //конец основного цикла программы


void Dimming() {
#ifdef pcgr
  static byte Secs;
  if (Secs >= 100)    {                                            //каждые 100 прерываний отправляем данные через serial
    Serial.println(buf);
    Secs = 1;                                                      //Начинаем считать с 1, т.к. в первый шаг мы только присваиваем значение, но не прибавляем.
  } else Secs++;
#endif
  if (reflowStatus == REFLOW_STATUS_ON) {
    OutPWR_TOP();
    OutPWR_BOTTOM();
  }
}

void OutPWR_TOP() {
  reg1 = Output1 + er1;                                            //pwr- задание выходной мощности в %, er- ошибка округления
  if (reg1 < 50) {
    out1 = LOW;
    er1 = reg1 ;                                                   //reg- переменная для расчетов
  } else {
    out1 = HIGH;
    er1 = reg1 - 100;
  }
  digitalWrite(RelayPin1, out1);                                   //пин через который осуществляется дискретное управление
}

void OutPWR_BOTTOM() {
  reg2 = Output2 + er2;                                            //pwr- задание выходной мощности в %, er- ошибка округления
  if (reg2 < 50) {
    out2 = LOW;
    er2 = reg2 ;                                                   //reg- переменная для расчетов
  } else {
    out2 = HIGH;
    er2 = reg2 - 100;
  }
  digitalWrite(RelayPin2, out2);                                   //пин через который осуществляется дискретное управление
}

void reflow_complete_ok() {
  if (reflowState == REFLOW_STATE_COMPLETE || alarmOn) {
    if (i < 2 && ESC_KEY.isPressed() == SB_NONE) {
      alarmOn = true;
      tone(buzzerPin, 1046, 100);
      delay(100);
      tone(buzzerPin, 1046, 100);
      delay(100);
      tone(buzzerPin, 1046, 100);
      delay(100);
      tone(buzzerPin, 1046, 100);
      delay(100);
      i++;
    } else {
      i = 0;
      alarmOn = false;
    }
  }
}

void setka() {
  for ( uint16_t str = 0; str <= (h / yc); str++) {
    tft.drawFastHLine(0, str * yc, w, ILI9341_GREEN);
  }
  for ( uint16_t col = 0; col <= (w / xc); col++) {
    tft.drawFastVLine(col * xc, 0, h, ILI9341_GREEN);
  }
}

void cls() {
  tft.fillRect(0, font5step_y, w, h - font5step_y, Bgcolor); //lcd.clear();
  //setka();
}

void CancelState() {
  beeper();
  updateScreen = true;
  SaveProfile();
  reflowState = REFLOW_STATE_IDLE;
}

int set_kPID(int kPID) {
#ifndef  resist_keyboard  
  switch (UP_KEY.isPressed()) {
    case SB_CLICK: {
        beeper();
        if (++kPID >= 255) kPID = 255;
      } break;
    case SB_AUTO_CLICK: {
        if (++kPID >= 255) kPID = 255;
      } break;
  }
  switch (DN_KEY.isPressed()) {
    case SB_CLICK: {
        beeper();
        if (--kPID <= 0) kPID = 0;
      } break;
    case SB_AUTO_CLICK: {
        if (--kPID <= 0) kPID = 0;
      } break;
  }
  return kPID;
#endif
}

int edit_kPID(int kPID) {
#ifndef  resist_keyboard
  if ( UP_KEY.isPressed() == SB_CLICK) {
    beeper();
    if (++kPID >= 255) {
      kPID = 255;
    }
    SaveProfile();
  }
  if ( DN_KEY.isPressed() == SB_CLICK) {
    beeper();
    if (--kPID <= 0) {
      kPID = 0;
    }
    SaveProfile();
  }
  return kPID;
#endif
}

void prn_gr() {
  tft.setTextWrap(true); // вернуть перенос строк
  tft.setTextSize(3);
  tft.setCursor(col(0), row(3) + 16);
  for (uint8_t i = 0; i < 127; i++) {
    tft.print(" ");
  }
  tft.setTextWrap(false); // отменить перенос строк
  tft.setTextSize(1); tft.setTextColor(ILI9341_BLACK, Bgcolor);
  tft.setCursor(col_1(1), row_1(10));
  tft.print(char(128)); tft.print("C");
  for (uint8_t i = 1; i < 7; i++) {
    tft.setCursor(0, h - i * 25);
    tft.print(i * 50);
    tft.drawFastHLine(18, h - i * 25, w - 18, ILI9341_YELLOW);
  }
  tft.drawRect(20, h - 150, w - 21, h - 91, ILI9341_MAGENTA);
}

void edit_pid() {
  tft.setTextSize(1); tft.setTextColor(ILI9341_RED, Bgcolor);
  tft.setCursor(col_1(30), row_1(6));
  tft.print(case0 ? "*" : " ");
  tft.print(char(31)); tft.print("P="); tft.print(profile.kp2);//низ
  tft.setCursor(col_1(30), row_1(7));
  tft.print(case1 ? "*" : " ");
  tft.print(char(31)); tft.print("I="); tft.print(profile.ki2); //низ
  tft.setCursor(col_1(30), row_1(8));
  tft.print(case2 ? "*" : " ");
  tft.print(char(31)); tft.print("D="); tft.print(profile.kd2);//низ
  tft.setCursor(col_1(40), row_1(6));
  tft.print(case3 ? "*" : " ");
  tft.print(char(30)); tft.print("P="); tft.print(profile.kp1);//верх
  tft.setCursor(col_1(40), row_1(7));
  tft.print(case4 ? "*" : " ");
  tft.print(char(30)); tft.print("I="); tft.print(profile.ki1); //верх
  tft.setCursor(col_1(40), row_1(8));
  tft.print(case5 ? "*" : " ");
  tft.print(char(30)); tft.print("D="); tft.print(profile.kd1);//верх

  if ( OK_KEY.isPressed() == SB_CLICK) {
    beeper();
    if ( ++ed_pid > 5 ) ed_pid = 0;
  }

  switch (ed_pid) {
    case 0:
      case0 = true;  case1 = false; case2 = false;
      case3 = false; case4 = false; case5 = false;
      profile.kp2 = edit_kPID(profile.kp2);
      break;

    case 1:
      case0 = false; case1 = true;  case2 = false;
      case3 = false; case4 = false; case5 = false;
      profile.ki2 = edit_kPID(profile.ki2);
      break;

    case 2:
      case0 = false; case1 = false; case2 = true;
      case3 = false; case4 = false; case5 = false;
      profile.kd2 = edit_kPID(profile.kd2);
      break;

    case 3:
      case0 = false; case1 = false; case2 = false;
      case3 = true;  case4 = false; case5 = false;
      profile.kp1 = edit_kPID(profile.kp1);
      break;

    case 4:
      case0 = false; case1 = false; case2 = false;
      case3 = false; case4 = true;  case5 = false;
      profile.ki1 = edit_kPID(profile.ki1);
      break;

    case 5:
      case0 = false; case1 = false; case2 = false;
      case3 = false; case4 = false; case5 = true;
      profile.kd1 = edit_kPID(profile.kd1);
      break;
  }
}

double max6675_read_temp (int ck, int cs, int so) {                            //---MAX6675 functions---//
  char i;
  int tmp = 0;
  digitalWrite(cs, LOW);//cs = 0;                                              //Stop a conversion in progress
  asm volatile (" nop"  "\n\t");
  for (i = 15; i >= 0; i--) {
    digitalWrite(ck, HIGH);
    asm volatile (" nop"  "\n\t");
    if ( digitalRead(so))
      tmp |= (1 << i);
    digitalWrite(ck, LOW);
    asm volatile (" nop"  "\n\t");
  }
  digitalWrite(cs, HIGH);
  if (tmp & 0x4) {
    return 255;
  } else
    return ((tmp >> 3)) * 0.25;
}

byte Pid1(double temp, byte ust, byte kP, byte kI, byte kd) {                 //----ПИД для ВИ
  byte out = 0;
  static float ed = 0;
  e1 = (ust - temp);                                                          //ошибка регулирования
  p1 =  (kP * e1);                                                            //П составляющая
  integra = (integra < i_min) ? i_min : (integra > i_max) ? i_max : integra + (kI * e1) / 1000; //И составляющая
  d1 = kd * (temp - ed); //Д составляющая
  ed = temp;
  out = (p1 + integra - d1 < 0) ? 0 : (p1 + integra - d1 > profile.pwr_TOP) ? profile.pwr_TOP : p1 + integra - d1;
  return out;
}

byte Pid2(double temp, int ust, byte kP, byte kI, byte kd) {                   //----ПИД для "шустрых" НИ на галогенках и им подобных излучателях
  byte out = 0;
  static float ed = 0;
  e2 = (ust - temp);                                                           //ошибка регулирования
  p2 =  (kP * e2);                                                             //П составляющая
  integra2 = (integra2 < i_min) ? i_min : (integra2 > i_max) ? i_max : integra2 + (kI * e2) / 1000; //И составляющая
  d2 = kd * (temp - ed) ;                                                      //Д составляющая
  ed = temp;
  out = (p2 + integra2 - d2 < 0) ? 0 : (p2 + integra2 - d2 > profile.pwr_BOTTOM) ? profile.pwr_BOTTOM : p2 + integra2 - d2;
  return out;
}

void loadProfile()  {                                             //чтение профиля из EEPROM
  for (byte j = 0; j <= SizeProfile - 1; j++) {
    EEPROM.get((currentProfile - 1)*SizeProfile, profile);
  }
}
void SaveProfile()    {                                           //сохранение текущего профиля
  for (byte j = 0; j <= SizeProfile - 1; j++) {
    EEPROM.put((currentProfile - 1)*SizeProfile, profile);
  }
}

void TempRead() {
  Input1 = max6675_read_temp (thermoCLK, thermoCStop, thermoSO);
  Input2 = max6675_read_temp (thermoCLK, thermoCSbott, thermoSO);
  tc1 = Input1;
  tc2 = Input2;
  tc1_gr = (tc1 < 255) ? tc1 : 255;
  tc2_gr = (tc2 < 255) ? tc2 : 255;
}

void HelloGuys() {                                                 //сообщение приветствия
  tft.begin();
  tft.setRotation(3);
  tft.setTextWrap(false);                                          //отменить перенос строк
  tft.fillRect(0, 0, w, h, Bgcolor);
  tft.setTextColor(ILI9341_NAVY);  tft.setTextSize(4);
  tft.setCursor(5, 5);
  tft.print("ARS NANO v1m5");
  tft.setCursor(col_5(2), row_5(2));
  tft.setTextColor(ILI9341_DARKGREEN);
  tft.print("REWORK");
  tft.setCursor(col_5(2), row_5(4));
  tft.print("STATION");
  tft.setTextColor(ILI9341_RED, Bgcolor);  tft.setTextSize(fontsize3);
}

void SongHello() {                                                 //Мелодия приветствия
  tone(buzzerPin, 523);
  delay(200);
  tone(buzzerPin, 659);
  delay(200);
  tone(buzzerPin, 784);
  delay(200);
  tone(buzzerPin, 1046);
  delay(200);
  noTone(buzzerPin);
}

void beeper() {                                                    //бузер при нажатии на кнопки (нервирует)
  if (beeper_on) {
    tone(buzzerPin, 1045, 100);
  }
}

void PushOk() {                                                  //обработка короткого нажатия Ok
  if (reflowState == REFLOW_STATE_IDLE) {                        //если коротко нажать Ok в IDLE
    tone(buzzerPin, 1045, 500);                                  //звуковой сигнал при старте профиля
    updateScreen = true;
#ifndef SerialPortPlotter                                        //если ПО irsp.exe от @Dmitrysh
    sprintf (buf, "SYNC\r\n");
    Serial.println(buf);
#endif
    curCount = 0;
    nextRead1 = millis();
    nextRead2 = millis();
    //digitalWrite(Ext_Fan, LOW);                                  //выключить охладитель платы
    //digitalWrite(Int_Fan, HIGH);                                 //включить охладитель симисторов и контроллера
    integra = 0;
    integra2 = 0;
    p1 = 0;
    p2 = 0;
    d1 = 0;
    d2 = 0;
    previousMillis = millis();
    reflowStatus = REFLOW_STATUS_ON;                             //статус "ПАЙКА" активируется
    reflowState = REFLOW_STATE_PRE_HEATER;                       //переходим в состояние "ПРЕДНАГРЕВ НИ"
    updateScreen = true;
  }
}

void PushOkLong() {                                              //обработка длинного нажатия Ok
  if (reflowState == REFLOW_STATE_STEP_RAMP || reflowState == REFLOW_STATE_STEP
      || reflowState == REFLOW_STATE_STEP_DWELL || reflowState == REFLOW_STATE_PRE_HEATER) { //ничего не делаем в этих состояниях
    return;
  }
  if (reflowState == REFLOW_STATE_IDLE) {                        //если держим долго Ok в режиме IDLE
    //reflowState = REFLOW_STATE_MENU_STEPS;                       //то заходим в меню настроек
    reflowState = REFLOW_STATE_MENU_BOTTOM_HEAT;                       //то заходим в меню настроек
    updateScreen = true;                                         //обновляем LCD
  } else {
    reflowState = REFLOW_STATE_IDLE;
    updateScreen = true;
    SaveProfile();
    //lcd.clear();
    //lcd.setCursor(4, 1);
    //lcd.print(F("Profile"));
    //lcd.setCursor(3, 2);
    //lcd.print(F("is Saved!"));
    //delay(2000);
  }
}

void PushUp() {                                                             //обработка короткого нажатия Up
  switch (reflowState) {                                                    //всё происходит внутри "машины состояний"
    case REFLOW_STATE_IDLE: {                                               //главное меню
        currentProfile ++;                                                   //увеличить № профиля на 1
        if (currentProfile >= 11) currentProfile = 1;                       //если текущий профиль > 14, нужно вернуться к профилю 1
        loadProfile();                                                      //вызов функции loadProfile для загрузки данных профиля из eeprom
        break;
      }
    case REFLOW_STATE_MENU_BOTTOM_HEAT: {                                   //устанавливаем температуру "Нижнего Нагревателя"
        profile.Setpoint2 ++;
        if (profile.Setpoint2 >= 34) profile.Setpoint2 = 8;                 //деленная на 10
        SaveProfile ();                                                     //вызов функции SaveProfile для записи данных профиля в eeprom
        break;
      }
    case REFLOW_STATE_MENU_BOTTOM_PWR:  {                                   //устанавливаем максимальную мощность "НИЖНЕГО НАГРЕВАТЕЛЯ"
        profile.pwr_BOTTOM ++;
        if (profile.pwr_BOTTOM >= 101) profile.pwr_BOTTOM = 1;
        SaveProfile ();                                                     //вызов функции SaveProfile для записи данных профиля в eeprom
        break;
      }
    case REFLOW_STATE_MENU_TOP_PWR: {                                   //устанавливаем минимальную мощность "ВЕРХНЕГО НАГРЕВАТЕЛЯ"
        profile.pwr_TOP ++;
        if (profile.pwr_TOP >= 101) profile.pwr_TOP = 1;
        SaveProfile ();                                                     //вызов функции SaveProfile для записи данных профиля в eeprom
        break;
      }
    case REFLOW_STATE_MENU_STEPS: {                                         //устанавливаем количество шагов профиля
        profile.profileSteps ++;                                             //увеличить число шагов профиля на 1
        if (profile.profileSteps >= 5) profile.profileSteps = 1;            //если число шагов > 4, установить число шагов = 1
        SaveProfile ();                                                     //вызов функции SaveProfile для записи данных профиля в eeprom
        break;
      }
    case REFLOW_STATE_MENU_STEP_TARGET: {                                   //устанавливаем температуру "Верхнего Нагревателя"
        profile.temperatureStep[editStep] ++;
        if (profile.temperatureStep[editStep] >= 256)  profile.temperatureStep[editStep] = 0;
        SaveProfile ();                                                     //вызов функции SaveProfile для записи данных профиля в eeprom
        break;
      }    
    case REFLOW_STATE_MENU_STEP_RAMP: {                                     //устанавливаем скорость нагрева "Верхним Нагревателем"
        profile.rampRateStep[editStep] ++;
        if (profile.rampRateStep[editStep] >= 31)  profile.rampRateStep[editStep] = 0;      //максимальная скорость роста температуры умноженная на 10
        SaveProfile ();                                                     //вызов функции SaveProfile для записи данных профиля в eeprom
        break;
      }
    case REFLOW_STATE_MENU_STEP_DWELL: {                                    //устанавливаем время перехода на следующий шаг (длительность шага)
        profile.dwellTimerStep[editStep] ++;
        if (profile.dwellTimerStep[editStep] >= 256)   profile.dwellTimerStep[editStep] = 0;
        SaveProfile ();                                                     //вызов функции SaveProfile для записи данных профиля в eeprom
        break;
      }
    case REFLOW_STATE_MENU_BOTTOM_P: {                                      //настройка "P" ПИД нижнего нагревателя
        profile.kp2 ++;
        if (profile.kp2 >= 256)  profile.kp2 = 0;
        SaveProfile ();                                                     //вызов функции SaveProfile для записи данных профиля в eeprom
        break;
      }
    case REFLOW_STATE_MENU_BOTTOM_I: {                                      //настройка "I" ПИД нижнего нагревателя
        profile.ki2 ++;
        if (profile.ki2 >= 256)  profile.ki2 = 0;
        SaveProfile ();                                                     //вызов функции SaveProfile для записи данных профиля в eeprom
        break;
      }
    case REFLOW_STATE_MENU_BOTTOM_D: {                                      //настройка "D" ПИД нижнего нагревателя
        profile.kd2 ++;
        if (profile.kd2 >= 256)  profile.kd2 = 0;
        SaveProfile ();                                                     //вызов функции SaveProfile для записи данных профиля в eeprom
        break;
      }
    case REFLOW_STATE_MENU_TOP_P: {                                         //настройка "P" ПИД верхнего нагревателя
        profile.kp1 ++;
        if (profile.kp1 >= 256)  profile.kp1 = 0;
        SaveProfile ();                                                     //вызов функции SaveProfile для записи данных профиля в eeprom
        break;
      }
    case REFLOW_STATE_MENU_TOP_I: {                                         //настройка "I" ПИД верхнего нагревателя
        profile.ki1 ++;
        if (profile.ki1 >= 256)  profile.ki1 = 0;
        SaveProfile ();                                                     //вызов функции SaveProfile для записи данных профиля в eeprom
        break;
      }
    case REFLOW_STATE_MENU_TOP_D: {                                         //настройка "D" ПИД верхнего нагревателя
        profile.kd1 ++;
        if (profile.kd1 >= 256)  profile.kd1 = 0;
        SaveProfile ();                                                     //вызов функции SaveProfile для записи данных профиля в eeprom
        break;
      }
  }
}

void PushDown() {                                                           //обработка короткого нажатия Down
  switch (reflowState) {                                                    //всё происходит внутри "машины состояний"
    case REFLOW_STATE_IDLE: {                                               //главное меню
        currentProfile --;                                                  //уменьшить № профиля на 1
        if (currentProfile <= 0) currentProfile = 10;                       //если № профиля <= 0, установить № профиля 14
        loadProfile();                                                      //вызов функции loadProfile для загрузки данных профиля из eeprom
        break;
      }
    case REFLOW_STATE_MENU_BOTTOM_HEAT: {                                   //устанавливаем температуру "Нижнего Нагревателя"
        profile.Setpoint2 --;
        if (profile.Setpoint2 <= 7) profile.Setpoint2 = 33;                 //деленная на 10
        SaveProfile ();                                                     //вызов функции SaveProfile для записи данных профиля в eeprom
        break;
      }     
   case REFLOW_STATE_MENU_BOTTOM_PWR: {                                    //устанавливаем максимальную мощность "НИЖНЕГО НАГРЕВАТЕЛЯ"
        profile.pwr_BOTTOM --;
        if (profile.pwr_BOTTOM <= 0) profile.pwr_BOTTOM = 100;
       SaveProfile ();                                                     //вызов функции SaveProfile для записи данных профиля в eeprom
        break;
     }
      case REFLOW_STATE_MENU_TOP_PWR: {                                   //устанавливаем минимальную мощность "ВЕРХНЕГО НАГРЕВАТЕЛЯ"
        profile.pwr_TOP --;
        if (profile.pwr_TOP >= 0) profile.pwr_TOP = 100;
        SaveProfile ();                                                     //вызов функции SaveProfile для записи данных профиля в eeprom
        break;
      }    
    case REFLOW_STATE_MENU_STEPS: {                                         //устанавливаем количество шагов профиля
        profile.profileSteps --;
        if (profile.profileSteps <= 0) profile.profileSteps = 4;
        SaveProfile ();                                                     //вызов функции SaveProfile для записи данных профиля в eeprom
        break;
      }    
    case REFLOW_STATE_MENU_STEP_TARGET: {                                   //устанавливаем температуру "Верхнего Нагревателя"
        profile.temperatureStep[editStep] --;
        if (profile.temperatureStep[editStep] <= 0)  profile.temperatureStep[editStep] = 256;
        if (editStep > 0 && profile.temperatureStep[editStep] < profile.temperatureStep[editStep - 1])
          profile.temperatureStep[editStep] = profile.temperatureStep[editStep - 1];
        SaveProfile ();                                                     //вызов функции SaveProfile для записи данных профиля в eeprom
        break;
      }
    case REFLOW_STATE_MENU_STEP_RAMP: {                                     //устанавливаем скорость нагрева "Верхним Нагревателем"
        profile.rampRateStep[editStep] --;
        if (profile.rampRateStep[editStep] <= 0)  profile.rampRateStep[editStep] = 30;      //минимальная скорость роста температуры умноженная на 10
        SaveProfile ();                                                     //вызов функции SaveProfile для записи данных профиля в eeprom
        break;
      }
    case REFLOW_STATE_MENU_STEP_DWELL: {                                    //устанавливаем время перехода на следующий шаг (длительность шага)
        profile.dwellTimerStep[editStep] --;
        if (profile.dwellTimerStep[editStep] <= 0)  profile.dwellTimerStep[editStep] = 0;
        SaveProfile ();                                                     //вызов функции SaveProfile для записи данных профиля в eeprom
        break;
      }
    case REFLOW_STATE_MENU_BOTTOM_P: {                                      //настройка "P" ПИД нижнего нагревателя
        profile.kp2 --;
        if (profile.kp2 < 1)  profile.kp2 = 255;
        SaveProfile ();                                                     //вызов функции SaveProfile для записи данных профиля в eeprom
        break;
      }
    case REFLOW_STATE_MENU_BOTTOM_I: {                                      //настройка "I" ПИД нижнего нагревателя
        profile.ki2 --;
        if (profile.ki2 < 1)  profile.ki2 = 255;
        SaveProfile ();                                                     //вызов функции SaveProfile для записи данных профиля в eeprom
        break;
      }
    case REFLOW_STATE_MENU_BOTTOM_D: {                                      //настройка "D" ПИД нижнего нагревателя
        profile.kd2 --;
        if (profile.kd2 < 1)   profile.kd2 = 255;
        SaveProfile ();                                                     //вызов функции SaveProfile для записи данных профиля в eeprom
        break;
      }
    case REFLOW_STATE_MENU_TOP_P: {                                         //настройка "P" ПИД верхнего нагревателя
        profile.kp1 --;
        if (profile.kp1 < 1)  profile.kp1 = 255;
        SaveProfile ();                                                     //вызов функции SaveProfile для записи данных профиля в eeprom
        break;
      }
    case REFLOW_STATE_MENU_TOP_I: {                                         //настройка "I" ПИД верхнего нагревателя
        profile.ki1 --;
        if (profile.ki1 < 1)  profile.ki1 = 255;
        SaveProfile ();                                                     //вызов функции SaveProfile для записи данных профиля в eeprom
        break;
      }
    case REFLOW_STATE_MENU_TOP_D: {                                         //настройка "D" ПИД верхнего нагревателя
        profile.kd1 --;
        if (profile.kd1 < 1)  profile.kd1 = 255;
        SaveProfile ();                                                     //вызов функции SaveProfile для записи данных профиля в eeprom
        break;
      }
  }
}

void PushRight() {                                                          //обработка короткого нажатия Right
  switch (reflowState) {
    case REFLOW_STATE_IDLE: {                                               //главное меню, управляем охладителем платы кнопкой ВПРАВО
   //     digitalWrite(Ext_Fan, HIGH);                                        //включить охладитель платы
        break;
      }
    case REFLOW_STATE_MENU_BOTTOM_HEAT: {                                   //устанавливаем температуру "Нижнего Нагревателя"
        updateScreen = true;
        reflowState = REFLOW_STATE_MENU_BOTTOM_PWR;
        break;
      }
    case REFLOW_STATE_MENU_BOTTOM_PWR:  {                                   //устанавливаем максимальную мощность "НИЖНЕГО НАГРЕВАТЕЛЯ"
        updateScreen = true;
        reflowState = REFLOW_STATE_MENU_TOP_PWR;
        break;
      }    
      case REFLOW_STATE_MENU_TOP_PWR: {
          updateScreen = true;
          reflowState = REFLOW_STATE_MENU_STEPS;
          break;
         }  
        case REFLOW_STATE_MENU_STEPS: {                                         //устанавливаем количество шагов профиля
        updateScreen = true;
        reflowState = REFLOW_STATE_MENU_STEP_TARGET;
        break;
      }
    case REFLOW_STATE_MENU_STEP_TARGET: {                                   //устанавливаем температуру "Верхнего Нагревателя"
        updateScreen = true;
        if (editStep + 1 == profile.profileSteps) {
          editStep = 0;
          reflowState = REFLOW_STATE_MENU_STEP_RAMP;
        }  else {
          editStep ++;
          if (profile.temperatureStep[editStep] < profile.temperatureStep[editStep - 1])
            profile.temperatureStep[editStep] = profile.temperatureStep[editStep - 1];
        }
        break;
      }
    case REFLOW_STATE_MENU_STEP_RAMP: {                                     //устанавливаем скорость нагрева "Верхним Нагревателем"
        updateScreen = true;
        if (editStep + 1 == profile.profileSteps) {
          editStep = 0;
          reflowState = REFLOW_STATE_MENU_STEP_DWELL;
        }  else editStep ++;
        break;
      }
    case REFLOW_STATE_MENU_STEP_DWELL: {                                    //устанавливаем время перехода на следующий шаг (длительность шага)
        updateScreen = true;
        if (editStep + 1 == profile.profileSteps)  {
          editStep = 0;
          reflowState = REFLOW_STATE_MENU_BOTTOM_P;
        }  else editStep ++;
        break;
      }
    case REFLOW_STATE_MENU_BOTTOM_P: {                                      //настройка "P" ПИД нижнего нагревателя
        updateScreen = true;
        reflowState = REFLOW_STATE_MENU_BOTTOM_I;
        break;
      }
    case REFLOW_STATE_MENU_BOTTOM_I: {                                      //настройка "I" ПИД нижнего нагревателя
        updateScreen = true;
        reflowState = REFLOW_STATE_MENU_BOTTOM_D;
        break;
      }
    case REFLOW_STATE_MENU_BOTTOM_D: {                                      //настройка "D" ПИД нижнего нагревателя
        updateScreen = true;
        reflowState = REFLOW_STATE_MENU_TOP_P;
        break;
      }
    case REFLOW_STATE_MENU_TOP_P: {                                         //настройка "P" ПИД верхнего нагревателя
        updateScreen = true;
        reflowState = REFLOW_STATE_MENU_TOP_I;
        break;
      }
    case REFLOW_STATE_MENU_TOP_I: {                                         //настройка "I" ПИД верхнего нагревателя
        updateScreen = true;
        reflowState = REFLOW_STATE_MENU_TOP_D;
        break;
      }
    case REFLOW_STATE_MENU_TOP_D: {                                         //настройка "D" ПИД верхнего нагревателя
        updateScreen = true;
        reflowState = REFLOW_STATE_IDLE;
        //lcd.clear();
        break;
      }
  }                                                                         //закрываем switch                                                                                        //конец обработки нажатия ВПРАВО
}                                                                           //закрываем PushRight

void PushLeft() {                                                           //обработка короткого нажатия Left
  if (reflowState == REFLOW_STATE_STEP_RAMP || reflowState == REFLOW_STATE_STEP
      || reflowState == REFLOW_STATE_STEP_DWELL || reflowState == REFLOW_STATE_PRE_HEATER) {
    reflowState = REFLOW_STATE_COMPLETE;
    reflowStatus = REFLOW_STATUS_OFF;
    return;
  }
  switch (reflowState) {
    //case REFLOW_STATE_IDLE: {                                                 //главное меню, управляем охладителем платы кнопкой ВПРАВО
    //    digitalWrite(Ext_Fan, LOW);                                           //выключить охладитель платы
    //    break;
      }
    case REFLOW_STATE_MENU_TOP_D: {                                           //устанавливаем количество шагов профиля
        updateScreen = true;
        reflowState = REFLOW_STATE_MENU_TOP_I;
        break;
      }
    case REFLOW_STATE_MENU_TOP_I: {                                           //устанавливаем температуру "Нижнего Нагревателя"
        updateScreen = true;
        reflowState = REFLOW_STATE_MENU_TOP_P;
        break;
      }
    case REFLOW_STATE_MENU_TOP_P:  {                                          //устанавливаем максимальную мощность "НИЖНЕГО НАГРЕВАТЕЛЯ"
        updateScreen = true;
        reflowState = REFLOW_STATE_MENU_BOTTOM_D;
        break;
      }
    case REFLOW_STATE_MENU_BOTTOM_D: {                                        //устанавливаем минимальную мощность "ВЕРХНЕГО НАГРЕВАТЕЛЯ"
        updateScreen = true;
        reflowState = REFLOW_STATE_MENU_BOTTOM_I;
        break;
      }
    case REFLOW_STATE_MENU_BOTTOM_I: {                                        //устанавливаем максимальную мощность "ВЕРХНЕГО НАГРЕВАТЕЛЯ"
        updateScreen = true;
        reflowState = REFLOW_STATE_MENU_BOTTOM_P;
        break;
      }
    case REFLOW_STATE_MENU_BOTTOM_P: {                                        //устанавливаем скорость нагрева "Верхним Нагревателем"
        updateScreen = true;
        reflowState = REFLOW_STATE_MENU_STEP_DWELL;
        lcd.setCursor(0, 3);
        lcd.print(F("                    "));
        break;
      }
    case REFLOW_STATE_MENU_STEP_DWELL: {                                      //устанавливаем температуру "Верхнего Нагревателя"
        updateScreen = true;
        if (editStep - 1 <= 0) {
          editStep = 0;
          reflowState = REFLOW_STATE_MENU_STEP_RAMP;
        }  else
          editStep --;
        break;
      }
    case REFLOW_STATE_MENU_STEP_RAMP: {                                      //настройка "P" ПИД нижнего нагревателя
        updateScreen = true;
        if (editStep - 1 <= 0) {
          editStep = 0;
          reflowState = REFLOW_STATE_MENU_STEP_TARGET;
        }  else
          editStep --;
        break;
      }
    case REFLOW_STATE_MENU_STEP_TARGET: {                                    //устанавливаем время перехода на следующий шаг (длительность шага)
        updateScreen = true;
        if (editStep - 1 <= 0) {
          editStep = 0;
          reflowState = REFLOW_STATE_MENU_STEPS;
          //}  else {
          //  editStep --;
          //if (profile.temperatureStep[editStep] < profile.temperatureStep[editStep - 1])
          //  profile.temperatureStep[editStep] = profile.temperatureStep[editStep - 1];
          //}
        }  else
          editStep --;
        break;
      }
    case REFLOW_STATE_MENU_STEPS: {                                          //настройка "D" ПИД верхнего нагревателя
        updateScreen = true;
        //reflowState = REFLOW_STATE_IDLE;
        reflowState = REFLOW_STATE_MENU_TOP_PWR;
        lcd.clear();
        break;
      }
    case REFLOW_STATE_MENU_TOP_PWR: {                                    //настройка "I" ПИД нижнего нагревателя
        updateScreen = true;
        reflowState = REFLOW_STATE_MENU_BOTTOM_PWR;
        break;
      }   
    case REFLOW_STATE_MENU_BOTTOM_PWR: {                                     //настройка "P" ПИД верхнего нагревателя
        updateScreen = true;
        reflowState = REFLOW_STATE_MENU_BOTTOM_HEAT;
        break;
      }
    case REFLOW_STATE_MENU_BOTTOM_HEAT: {                                    //настройка "I" ПИД верхнего нагревателя
        updateScreen = true;
        reflowState = REFLOW_STATE_IDLE;
        break;
      }

  }                                                                           //закрываем switch                                                                                        //конец обработки нажатия LEFT
}                                                                             //закрываем PushLeft

void PushLeftLong() {                                                         //обработка длинного нажатия Left
  if (reflowState == REFLOW_STATE_IDLE) {                                     //держим долго Left в режиме IDLE
    reflowState = REFLOW_STATE_PROFILE_INIT;                                  //и запускается инициализация предустановленного профиля
  }
}

void PushUpLong() {                               //обработка длинного нажатия Up (пока в резерве)

}

void PushDownLong() {                             //обработка длинного нажатия Down (пока в резерве)

}

void PushRightLong() {                            //обработка длинного нажатия Right (пока в резерве)

}  

// colors of ILI9341
// ILI9341_BLACK       0x0000      /*   0,   0,   0 */
// ILI9341_NAVY        0x000F      /*   0,   0, 128 */
// ILI9341_DARKGREEN   0x03E0      /*   0, 128,   0 */
// ILI9341_DARKCYAN    0x03EF      /*   0, 128, 128 */
// ILI9341_MAROON      0x7800      /* 128,   0,   0 */
// ILI9341_PURPLE      0x780F      /* 128,   0, 128 */
// ILI9341_OLIVE       0x7BE0      /* 128, 128,   0 */
// ILI9341_LIGHTGREY   0xC618      /* 192, 192, 192 */
// ILI9341_DARKGREY    0x7BEF      /* 128, 128, 128 */
// ILI9341_BLUE        0x001F      /*   0,   0, 255 */
// ILI9341_GREEN       0x07E0      /*   0, 255,   0 */
// ILI9341_CYAN        0x07FF      /*   0, 255, 255 */
// ILI9341_RED         0xF800      /* 255,   0,   0 */
// ILI9341_MAGENTA     0xF81F      /* 255,   0, 255 */
// ILI9341_YELLOW      0xFFE0      /* 255, 255,   0 */
// ILI9341_WHITE       0xFFFF      /* 255, 255, 255 */
// ILI9341_ORANGE      0xFD20      /* 255, 165,   0 */
// ILI9341_GREENYELLOW 0xAFE5      /* 173, 255,  47 */
// ILI9341_PINK        0xF81F
